@@ -1,5 +1,5 @@
 
-# $Id: Wishlist.pm,v 2.6 2009-08-26 22:04:49 Martin Exp $
+# $Id: Wishlist.pm,v 2.9 2013-12-14 20:42:57 Martin Exp $
 
 package WWW::Amazon::Wishlist;
 
@@ -33,7 +33,7 @@ require Exporter;
 );
 
 our
-$VERSION = do { my @r = (q$Revision: 2.6 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
+$VERSION = do { my @r = (q$Revision: 2.9 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
 
 
 =pod
@@ -151,18 +151,15 @@ my $USER_AGENT = 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)';
 
 sub get_list
   {
-  my ($id, $uk, $test) = @_;
-  if (! defined $id)
-    {
-    croak "No ID given to get_list function\n";
-    return undef;
-    } # if
-  # note to self ... should we UC the id? Nahhhh. Not yet.
-  # Default is amazon.com:
-  $uk |= 0;
-  $test |= 0;
+  # Required arg = wishlist ID:
+  my $id = shift || croak "No ID given to get_list() function\n";
+  # Optional arg = whether we're accessing the UK site.  Default is "no":
+  my $uk = shift || 0;
+  # Optional arg = turn on debugging:
+  my $test = shift || DEBUG;
+  # Note to self ... should we UC the id? Nahhhh. Not yet.
   # fairly self explanatory
-  my $domain = ($uk)? "co.uk" : "com";
+  my $domain = ($uk) ? "co.uk" : "com";
   # set up some variables
   my $iPage = 1;
   my @items;
@@ -174,30 +171,42 @@ sub get_list
     "http://www.amazon.com/gp/registry/wishlist/$id/?page=$iPage";
     # This is a typical complete .com URL as of 2008-12:
     # http://www.amazon.com/gp/registry/wishlist/2O4B95NPM1W3L
-    DEBUG_HTML && print STDERR " DDD fetching wishlist for $id, page $iPage...\n";
+    DEBUG_HTML && warn " DDD fetching wishlist for $id, page $iPage...\n";
     # Don't overwhelm the server:
     sleep(3) if (1 < $iPage);
     my $content = _fetch_page($url, $domain);
+    if (DEBUG_HTML == 88)
+      {
+      warn $content;
+      exit 88;
+      } # if
     # As of 2009-08, Amazon returns HTML with MISSING BRACKETS:
     $content =~ s/(<tbody\s[^>\r\n]+)(\s+<)/$1>\n$2/g;
     # There seems to be a bug in HTML::TreeBuilder that causes
     # abutting tags to be skpped!?!
     $content =~ s!><!> <!g;
-    if (0)
+    if (9 < $test)
       {
       eval "use File::Slurp";
       write_file('PAGES/fetched.html', $content);
       exit 88;
       } # if
     my $iLen = length($content);
-    # print STDERR " DDD fetched $iLen bytes.\n";
-    return undef unless ($content);
-    # print STDERR " DDD call _extract()\n";
-    my $result = _extract($uk, $content);
+    # warn " DDD fetched $iLen bytes.\n";
+    my $result = _extract($uk, $content, $test);
     # print Dumper($result);
     # exit 88;
-    last INFINITE if (! defined $result);
-    last INFINITE if (! ref $result->{items});
+    if (! defined $result)
+      {
+      DEBUG && warn " WWW _extract() returned nothing\n";
+      last INFINITE;
+      } # if
+    if (! ref $result->{items})
+      {
+      # Probably an empty wish list
+      DEBUG && warn " WWW _extract() returned no items\n";
+      last INFINITE;
+      } # if
  ITEM:
     foreach my $item (@{$result->{items}})
       {
@@ -218,42 +227,41 @@ sub get_list
         } # if
       push @items, $item;
       } # foreach ITEM
-    # For debug purposes:
-    last INFINITE if $test;
     my $sURLNext = $result->{next};
+    my $iNext = 0;
     if (! defined $sURLNext)
       {
-      # DEBUG && print STDERR " DDD content===$content===\n";
+      # DEBUG && warn " DDD content===$content===\n";
       # exit 88;
       # Use brute force to find it:
-      if ($content =~ m!(&(amp;)?page=\d+)">\s*(<[^>]+>)?Next!)
+      if ($content =~ m!([;&]page=\d+)">\s*(<[^>]+>)?Next!)
         {
-        DEBUG && print STDERR " DDD found next URL with brute force\n";
+        DEBUG && warn " DDD found next URL with brute force\n";
         $sURLNext = $1;
+        $iNext = $2;
         } # if
       } # if
     # Paranoia:
     if (! defined $sURLNext)
       {
-      DEBUG && print STDERR " WWW did not find next url\n";
+      DEBUG && warn " WWW did not find next url\n";
       last INFINITE;
       } # if
-    my $iNext = 0;
     if ($sURLNext !~ m/[;&]page=(\d+)/)
       {
-      DEBUG && print STDERR " WWW next url =$sURLNext= does not contain page#\n";
+      DEBUG && warn " WWW next url =$sURLNext= does not contain page#\n";
       last INFINITE;
       } # if
     $iNext = $1;
     # More paranoia:
     if ($iNext <= $iPage)
       {
-      DEBUG && print STDERR " WWW next url page=$iNext is not greater than current page=$iPage\n";
+      DEBUG && warn " WWW next url page=$iNext is not greater than current page=$iPage\n";
       last INFINITE;
       } # if
     # ...and update:
     $iPage = $iNext;
-    } # while
+    } # while INFINITE
   return @items;
   } # get_list
 
@@ -274,54 +282,49 @@ sub _fetch_page
 
     # set up the UA
     my $ua = new LWP::UserAgent( keep_alive => 1, timeout => 30, agent => $USER_AGENT, );
-
     # setting it in the 'new' seems not to work sometimes
     $ua->agent($USER_AGENT);
     # for some reason this makes stuff work
     $ua->max_redirect( 0 );
-
-
     # make a full set of headers
     my $h = new HTTP::Headers(
-                'Host'            => "www.amazon.$domain",
-                'Referer'         => $url,
-                'User-Agent'      => $USER_AGENT,
-                'Accept'          => 'text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,video/x-mng,image/png,image/jpeg,image/gif;q=0.2,*/*;q=0.1',
-                'Accept-Language' => 'en-us,en;q=0.5',
-                'Accept-Charset'  => 'ISO-8859-1,utf-8;q=0.7,*;q=0.7',
-                #'Accept-Encoding' => 'gzip,deflate',
-                'Keep-Alive'      =>  '300',
-                'Connection'      =>  'keep-alive',
-    );
+                              'Host'            => "www.amazon.$domain",
+                              'Referer'         => $url,
+                              'User-Agent'      => $USER_AGENT,
+                              'Accept'          => 'text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,video/x-mng,image/png,image/jpeg,image/gif;q=0.2,*/*;q=0.1',
+                              'Accept-Language' => 'en-us,en;q=0.5',
+                              'Accept-Charset'  => 'ISO-8859-1,utf-8;q=0.7,*;q=0.7',
+                              #'Accept-Encoding' => 'gzip,deflate',
+                              'Keep-Alive'      =>  '300',
+                              'Connection'      =>  'keep-alive',
+                             );
     $h->referer("$url");
-
-
-
     my $request  =  HTTP::Request->new ( 'GET', $url, $h );
     my $response;
-
     my $times = 0;
-
     # LWP should be able to do this but seemingly fails sometimes
-    while ($times++<3) {
-        $response =  $ua->request($request);
-        last if $response->is_success;
-        if ($response->is_redirect) {
-               $url = $response->header("Location");
-            #$h->header("Referer", $url); 
-            $h->referer("$url");
-            $request  =  HTTP::Request->new ( 'GET', $url, $h );
-        }
-    }
-
-    if (!$response->is_success)     {
-        croak "Failed to retrieve $url";
-        return undef;
-    }
-
-    return $response->content;
-
-} # _fetch_page
+    while ($times++<3)
+      {
+      $response =  $ua->request($request);
+      last if $response->is_success;
+      if ($response->is_redirect)
+        {
+        $url = $response->header("Location");
+        #$h->header("Referer", $url); 
+        $h->referer("$url");
+        $request  =  HTTP::Request->new ( 'GET', $url, $h );
+        } # if
+      } # while
+    if (!$response->is_success)
+      {
+      croak "Failed to retrieve $url";
+      return undef;
+      } # if
+    my $s = $response->content;
+    # Clean the CRAP off the page:
+    $s =~ s!<script>.+?</script>!!gs;
+    return $s;
+    } # _fetch_page
 
 # This is the HTML parsing version written by Martin Thurn:
 
@@ -331,91 +334,115 @@ sub _extract
   my $iUK = shift || 0;
   # Required arg2 = the HTML contents of the webpage:
   my $s = shift || '';
-  DEBUG_HTML && print STDERR " DDD start _extract()\n";
-  my $rh;
+  # Optional arg = debugging level:
+  my $iDebug = shift || 0;
+  DEBUG_HTML && warn " DDD start _extract()\n";
+  my $rh = {};
   my $oTree = new HTML::TreeBuilder;
   $oTree->parse($s);
   $oTree->eof;
-  my @aoSPAN = $iUK ? $oTree->look_down(_tag => 'td',
-                                        class => 'small',
+  my @aoSPAN = $iUK ? $oTree->look_down(_tag => 'div',
+                                        class => 'lineItemGroup',
                                        )
-                    : $oTree->look_down(_tag => 'span',
-                                        class => 'small productTitle',
+                    : $oTree->look_down(_tag => 'div',
+                                        class => 'a-text-left a-fixed-left-grid-col a-col-right',
                                        );
  SPAN_TAG:
   foreach my $oSPAN (@aoSPAN)
     {
     next SPAN_TAG unless ref $oSPAN;
-    DEBUG_HTML && print STDERR " DDD found TD...\n";
-    my $oA = $oSPAN->look_down(_tag => 'a');
-    next SPAN_TAG unless ref $oA;
-    DEBUG_HTML && print STDERR " DDD   found A...\n";
-    my $sTitle = $oA->as_text;
-    # Strip leading whitespace:
-    $sTitle =~ s!\A\s+!!;
-    # Strip trailing whitespace:
-    $sTitle =~ s!\s+\Z!!;
-    next SPAN_TAG unless ($sTitle =~ m!\S!);
-    # Strip out zero-width spaces scattered about randomly in item titles
-    $sTitle =~ s/\x{200b}//g;
-    DEBUG_HTML && print STDERR " DDD found item named '$sTitle'\n";
-    my $sURL = $oA->attr('href');
-    DEBUG_HTML && print STDERR " DDD   URL ==$sURL==\n";
-    my $sASIN = '';
-    if (
-        ($sURL =~ m!/detail(?:/offer-listing)?/-/(.+?)/ref!)
-        ||
-        ($sURL =~ m!/gp/product/(.+?)/ref!)
-        ||
-        ($sURL =~ m!/dp/(.+?)/ref!)
-       )
+    DEBUG_HTML && warn " DDD found toplevel item tagset\n";
+    if (9 < DEBUG_HTML)
       {
-      $sASIN = $1;
+      my $s = $oSPAN->as_HTML;
+      warn " DDD ==$s==\n";
       } # if
-    else
+    my $sASIN = q{};
+    my $sName = q{};
+    my $sTitle = q{};
+    my @aoA = $oSPAN->look_down(_tag => 'a');
+    DEBUG_HTML && warn sprintf(" DDD   contains %d <A> tags\n", scalar(@aoA));
+ A_TAG:
+    foreach my $oA (@aoA)
       {
-      DEBUG_HTML && print STDERR " EEE   url does not contain asin\n";
-      }
-    DEBUG_HTML && print STDERR " DDD   ASIN ==$sASIN==\n";
-    # Grab the smallest-containing ancestor of this item:
-    my $oParent = $oSPAN->look_up(_tag => 'tbody',
-                                 class => 'itemWrapper',
-                                );
-    if (! ref $oParent)
+      next A_TAG if ! ref $oA;
+      my $sA = $oA->as_HTML;
+      DEBUG_HTML && warn " DDD   try A\n";
+      if (9 < DEBUG_HTML)
+        {
+        warn " DDD ==$sA==\n";
+        } # if
+      $sTitle = $oA->attr('title') || $oA->as_text;
+      # Strip leading whitespace:
+      $sTitle =~ s!\A\s+!!;
+      # Strip trailing whitespace:
+      $sTitle =~ s!\s+\Z!!;
+      # Ignore empty (image-only) tags:
+      next A_TAG if ($sTitle !~ m/\S/);
+      # Strip out zero-width spaces scattered about randomly in item titles
+      $sTitle =~ s/\x{200b}//g;
+      DEBUG_HTML && warn " DDD found item named '$sTitle'\n";
+      next A_TAG if ($sTitle eq 'Universal Wish List Button');
+      next A_TAG if ($sTitle eq 'Buying this gift elsewhere?');
+      my $sURL = $oA->attr('href');
+      DEBUG_HTML && warn " DDD   URL ==$sURL==\n";
+      if (
+          ($sURL =~ m!/detail(?:/offer-listing)?/-/(.+?)/ref!)
+          ||
+          ($sURL =~ m!/gp/product/(.+?)/ref!)
+          ||
+          ($sURL =~ m!/dp/(.+?)/ref!)
+         )
+        {
+        # It's a match!
+        $sASIN = $1;
+        last A_TAG;
+        } # if
+      else
+        {
+        DEBUG_HTML && warn " EEE   url does not contain asin\n";
+        }
+      } # foreach A_TAG
+    DEBUG_HTML && warn " DDD   ASIN ==$sASIN==\n";
+    if ($sASIN eq q{})
       {
-      DEBUG_HTML && print STDERR " WWW did not find ancestor TBODY\n";
       next SPAN_TAG;
       } # if
-    my $sParent = $oParent->as_text;
-    my $sParentHTML = $oParent->as_HTML;
-    print STDERR " DDD   parent TR HTML ==$sParentHTML==\n" if (4 < DEBUG_HTML);
-    print STDERR " DDD   parent TR ==$sParent==\n" if (4 < DEBUG_HTML);
-    # Find the quantity desired, and the priority.  These are the defaults:
-    my $iDesired = 1;
-    my $sPriority = 'medium';
-    # my @aoSPAN = $oParent->look_down(_tag => 'span',
-    #                                  class => 'wl-iter-heading',
-    #                                 );
-    my @aoSPAN = $oParent->look_down(_tag => 'td',
-                                     class => 'tiny',
-                                    );
- SPAN_TAG:
-    foreach my $oSPAN (@aoSPAN)
+    # Grab the smallest-containing ancestor of this item:
+    my $oParent = $iUK
+                ? $oSPAN->look_up(_tag => 'tbody',
+                                  class => 'itemWrapper',
+                                 )
+                : $oSPAN;
+    if (! ref $oParent)
       {
-      next SPAN_TAG unless ref $oSPAN;
+      DEBUG_HTML && warn " WWW did not find ancestor TBODY\n";
+      next SPAN_TAG;
+      } # if
+    my $sParentHTML = $oParent->as_HTML;
+    DEBUG_HTML && warn " DDD   parent HTML ==$sParentHTML==\n";
+    my $sParent = $oParent->as_text;
+    # Manual text clean-up:
+    $sParent =~ s/(DESIRED|RECEIVED|PRIORITY)/;  $1: /g;
+    DEBUG_HTML && warn " DDD   parent text ==$sParent==\n";
+    my $iDesired = _match_desired($sParent);
+    DEBUG_HTML && warn " DDD     desired set to =$iDesired=\n";
+    my $sPriority = _match_priority($sParent);
+    DEBUG_HTML && warn " DDD     priority set to =$sPriority=\n";
+    my @aoTDtiny = $oParent->look_down(_tag => 'td',
+                                       class => 'tiny',
+                                      );
+ QUANT_TAG:
+    foreach my $oSPAN (@aoTDtiny)
+      {
+      next QUANT_TAG unless ref $oSPAN;
       my $sSpan = $oSPAN->as_text;
-      DEBUG_HTML && print STDERR " DDD   span=$sSpan=\n";
-      if ($sSpan =~ m'DESIRED(\d+)')
-        {
-        $iDesired = $1;
-        DEBUG_HTML && print STDERR " DDD     desired=$iDesired=\n";
-        } # if
-      if ($sSpan =~ m'PRIORITY(.+)\z')
-        {
-        $sPriority = $1;
-        DEBUG_HTML && print STDERR " DDD     priority=$sPriority=\n";
-        } # if
-      } # foreach SPAN_TAG
+      DEBUG_HTML && warn " DDD   TDtiny=$sSpan=\n";
+      $sPriority ||= _match_priority($sSpan);
+      DEBUG_HTML && warn " DDD     priority set to =$sPriority=\n";
+      $iDesired ||= _match_desired($sSpan);
+      DEBUG_HTML && warn " DDD     desired set to =$iDesired=\n";
+      } # foreach QUANT_TAG
     if (! $iDesired || ! $sPriority)
       {
       # See if they are encoded in a FORM:
@@ -423,70 +450,74 @@ sub _extract
       if ($sParentHTML =~ m!<option selected="yes" value=([-0-9]+)>!)
         {
         $sPriority = $1;
-        DEBUG_HTML && print STDERR " DDD   priority=$sPriority=\n";
+        DEBUG_HTML && warn " DDD     priority set to =$sPriority=\n";
         } # if
       else
         {
-        DEBUG_HTML && print STDERR " WWW   did not find <option> for priority\n";
+        DEBUG_HTML && warn " WWW   did not find <option> for priority\n";
         }
       # Find the quantity desired:
       if ($sParentHTML =~ m!<input class="tiny" name="requestedQty.+?" size=\d+ type="text" value=(\d+)>!)
         {
         $iDesired = $1;
-        DEBUG_HTML && print STDERR " DDD   desired=$iDesired=\n";
+        DEBUG_HTML && warn " DDD     desired set to =$iDesired=\n";
         } # if
       else
         {
-        DEBUG_HTML && print STDERR " WWW   did not find <input> for desired-quantity\n";
+        DEBUG_HTML && warn " WWW   did not find <input> for desired-quantity\n";
         }
       } # if
+    # Put in default values if we never found them:
+    $sPriority ||= 'medium';
+    DEBUG_HTML && warn " DDD     priority set to =$sPriority=\n";
+    $iDesired ||= 1;
     # Find the date added:
     my $sDate = '';
     if ($sParentHTML =~ m!>added\s+(.+?)<!)
       {
       $sDate = $1;
-      DEBUG_HTML && print STDERR " DDD   date=$sDate=\n";
+      DEBUG_HTML && warn " DDD   date=$sDate=\n";
       } # if
     else
       {
-      DEBUG_HTML && print STDERR " WWW   did not find text for date-added\n";
+      DEBUG_HTML && warn " WWW   did not find text for date-added\n";
       }
 
     # Find the "author" of this item:
-    my @aoTD;
+    my @aoTDauthor;
     if ($iUK)
       {
-      @aoTD = $oParent->look_down(_tag => 'td',
-                                  class => 'small',
-                                 );
+      @aoTDauthor = $oParent->look_down(_tag => 'td',
+                                        class => 'small',
+                                       );
       }
     else
       {
-      @aoTD = $oParent->look_down(_tag => 'span',
-                                  sub
-                                    {
-                                    my $sHtml = $_[0]->as_HTML;
-                                    # DEBUG_HTML && print STDERR " DDD   try oTDauthor span==$sHtml==\n";
-                                    my $s = $_[0]->attr('class') || q{};
-                                    $s =~ m'BYLINE'i;
-                                    },
-                                 );
+      @aoTDauthor = $oParent->look_down(_tag => 'span',
+                                        sub
+                                          {
+                                          my $sHtml = $_[0]->as_HTML;
+                                          # DEBUG_HTML && warn " DDD   try oTDauthor span==$sHtml==\n";
+                                          my $s = $_[0]->attr('class') || q{};
+                                          $s =~ m'BYLINE'i;
+                                          },
+                                       );
       } # else
     my $sAuthor = '';
- TD_TAG:
-    foreach my $oTD (@aoTD)
+ AUTHOR_TAG:
+    foreach my $oTD (@aoTDauthor)
       {
-      next TD_TAG unless ref $oTD;
+      next AUTHOR_TAG unless ref $oTD;
       my $s = $oTD->as_HTML;
-      DEBUG_HTML && print STDERR " DDD   try oTDauthor==$s==\n";
+      DEBUG_HTML && warn " DDD   try oTDauthor==$s==\n";
       $s = $oTD->as_text;
       if ($s =~ s!\A\s*(by|~)\s+!!)
         {
         $sAuthor = $s;
-        last TD_TAG;
+        last AUTHOR_TAG;
         } # if
-      } # foreach TD_TAG
-    DEBUG_HTML && print STDERR " DDD   author=$sAuthor=\n";
+      } # foreach AUTHOR_TAG
+    DEBUG_HTML && warn " DDD   author=$sAuthor=\n";
     # Find the price of this item:
     my $sPrice = '';
     my $oTDprice = $oParent->look_down(_tag => 'span',
@@ -498,8 +529,8 @@ sub _extract
                                       );
     if (! ref $oTDprice)
       {
-      DEBUG_HTML && print STDERR " WWW did not find TD for price\n";
-      # print STDERR $oParent->as_HTML;
+      DEBUG_HTML && warn " WWW did not find TD for price\n";
+      # warn $oParent->as_HTML;
       # exit 88;
       # next SPAN_TAG;
       } # if
@@ -510,7 +541,9 @@ sub _extract
         {
         $sPrice = $1;
         } # if
-      DEBUG_HTML && print STDERR " DDD   price=$sPrice=\n";
+      $sPrice =~ s!\A\s+!!;
+      $sPrice =~ s!\s+\Z!!;
+      DEBUG_HTML && warn " DDD   price=$sPrice=\n";
       } # else
     # Add this item to the result set:
     my %hsItem = (
@@ -523,7 +556,7 @@ sub _extract
                   title => $sTitle,
                   # type => $sType,
                  );
-    DEBUG_HTML && print STDERR Dumper(\%hsItem);
+    DEBUG_HTML && warn Dumper(\%hsItem);
     push @{$rh->{items}}, \%hsItem;
     # All done with this item:
     $oParent->detach;
@@ -534,21 +567,42 @@ sub _extract
                              sub
                                {
                                my $s = $_[0]->as_text || q{};
-                               $s =~ m/\A\s*Next/;
+                               DEBUG_HTML && warn " DDD   try next <A> ==$s==\n";
+                               $s =~ m/\A\s*NEXT/i;
                                },
                             );
   if (ref $oA)
     {
     $rh->{next} = $oA->attr('href');
-    DEBUG_HTML && print STDERR " DDD raw next URL is ==$rh->{next}==\n";
+    DEBUG_HTML && warn " DDD raw next URL is ==$rh->{next}==\n";
     } # if
   else
     {
-    DEBUG_HTML && print STDERR " DDD did not find next URL\n";
+    DEBUG_HTML && warn " DDD did not find next URL\n";
     }
   return $rh;
   } # _extract
 
+sub _match_priority
+    {
+    my $s = shift || return;
+    if ($s =~ m'.+PRIORITY:?\s*(\w+?)(\s|\z)'i)
+      {
+      return lc $1;
+      } # if
+    return;
+    } # _match_priority
+
+sub _match_desired
+      {
+      my $s = shift || return;
+      if ($s =~ m'(?:DESIRED|WANTS):?\s*(\d+)'i)
+        {
+        return lc $1;
+        } # if
+      return;
+      } # _match_desired
+
 1;
 
 
